@@ -53,6 +53,10 @@ struct _WnckScreenPrivate
   
   guint update_handler;  
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+  SnDisplay *sn_display;
+#endif
+  
   guint showing_desktop : 1;
   
   /* if you add flags, be sure to set them
@@ -288,12 +292,32 @@ wnck_screen_finalize (GObject *object)
   g_list_free (screen->priv->workspaces);
 
   screens[screen->priv->number] = NULL;
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  sn_display_unref (screen->priv->sn_display);
+#endif
   
   g_free (screen->priv);
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+static void
+sn_error_trap_push (SnDisplay *display,
+                    Display   *xdisplay)
+{
+  gdk_error_trap_push ();
+}
+
+static void
+sn_error_trap_pop (SnDisplay *display,
+                   Display   *xdisplay)
+{
+  gdk_error_trap_pop ();
+}
+#endif /* HAVE_STARTUP_NOTIFICATION */
+
 static void
 wnck_screen_construct (WnckScreen *screen,
                        int         number)
@@ -303,6 +327,12 @@ wnck_screen_construct (WnckScreen *screen,
   screen->priv->xscreen = ScreenOfDisplay (gdk_display, number);
   screen->priv->number = number;
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+  screen->priv->sn_display = sn_display_new (gdk_display,
+                                             sn_error_trap_push,
+                                             sn_error_trap_pop);
+#endif
+  
   screen->priv->bg_pixmap = None;
   
   _wnck_select_input (screen->priv->xroot,
@@ -353,6 +383,18 @@ wnck_screen_get (int index)
   return screens[index];
 }
 
+WnckScreen*
+_wnck_screen_get_existing (int number)
+{
+  g_return_val_if_fail (gdk_display != NULL, NULL);
+  g_return_val_if_fail (number < ScreenCount (gdk_display), NULL);
+
+  if (screens != NULL)
+    return screens[number];
+  else
+    return NULL;
+}
+
 WnckScreen*
 wnck_screen_get_default (void)
 {
@@ -1380,6 +1422,12 @@ _wnck_screen_get_xscreen (WnckScreen *screen)
   return screen->priv->xscreen;
 }
 
+int
+_wnck_screen_get_number (WnckScreen *screen)
+{
+  return screen->priv->number;
+}
+
 int
 wnck_screen_try_set_workspace_layout (WnckScreen *screen,
                                       int         current_token,
@@ -1427,3 +1475,13 @@ wnck_screen_toggle_showing_desktop (WnckScreen *screen,
   _wnck_toggle_showing_desktop (screen->priv->xscreen,
                                 show);
 }
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+SnDisplay*
+_wnck_screen_get_sn_display (WnckScreen *screen)
+{
+  g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
+  
+  return screen->priv->sn_display;
+}
+#endif /* HAVE_STARTUP_NOTIFICATION */
