@@ -662,6 +662,7 @@ filter_func (GdkXEvent  *gdkxevent,
              gpointer    data)
 {
   XEvent *xevent = gdkxevent;
+  int i;
   
   switch (xevent->type)
     {
@@ -705,6 +706,26 @@ filter_func (GdkXEvent  *gdkxevent,
         _wnck_desktop_layout_manager_process_event (xevent);
       }
       break;
+
+    case ClientMessage:
+#ifdef HAVE_STARTUP_NOTIFICATION
+      /* We're cheating as officially libsn requires
+       * us to send all events through sn_display_process_event
+       */
+      i = 0;
+      while (i < ScreenCount (gdk_display))
+        {
+          WnckScreen *s;
+
+          s = _wnck_screen_get_existing (i);
+          if (s != NULL)
+            sn_display_process_event (_wnck_screen_get_sn_display (s),
+                                      xevent);
+          
+          ++i;
+        }
+#endif /* HAVE_STARTUP_NOTIFICATION */
+      break;
     }
   
   return GDK_FILTER_CONTINUE;
@@ -793,6 +814,68 @@ _wnck_close (Screen *screen,
 	      &xev); 
 }
 
+#define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
+#define _NET_WM_MOVERESIZE_SIZE_TOP          1
+#define _NET_WM_MOVERESIZE_SIZE_TOPRIGHT     2
+#define _NET_WM_MOVERESIZE_SIZE_RIGHT        3
+#define _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT  4
+#define _NET_WM_MOVERESIZE_SIZE_BOTTOM       5
+#define _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT   6
+#define _NET_WM_MOVERESIZE_SIZE_LEFT         7
+#define _NET_WM_MOVERESIZE_MOVE              8
+#define _NET_WM_MOVERESIZE_SIZE_KEYBOARD     9
+#define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10
+
+void
+_wnck_keyboard_move (Screen *screen,
+                     Window  xwindow)
+{
+  XEvent xev;
+  
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = gdk_display;
+  xev.xclient.window = xwindow;
+  xev.xclient.message_type = _wnck_atom_get ("_NET_WM_MOVERESIZE");
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = 0; /* unused */
+  xev.xclient.data.l[1] = 0; /* unused */
+  xev.xclient.data.l[2] = _NET_WM_MOVERESIZE_MOVE_KEYBOARD;
+  xev.xclient.data.l[3] = 0; /* unused */
+
+  XSendEvent (gdk_display,
+              RootWindowOfScreen (screen),
+              False,
+              SubstructureRedirectMask | SubstructureNotifyMask,
+              &xev); 
+}
+
+void
+_wnck_keyboard_size (Screen *screen,
+                     Window  xwindow)
+{
+  XEvent xev;
+  
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = gdk_display;
+  xev.xclient.window = xwindow;
+  xev.xclient.message_type = _wnck_atom_get ("_NET_WM_MOVERESIZE");
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = 0; /* unused */
+  xev.xclient.data.l[1] = 0; /* unused */
+  xev.xclient.data.l[2] = _NET_WM_MOVERESIZE_SIZE_KEYBOARD;
+  xev.xclient.data.l[3] = 0; /* unused */
+
+  XSendEvent (gdk_display,
+              RootWindowOfScreen (screen),
+              False,
+              SubstructureRedirectMask | SubstructureNotifyMask,
+              &xev); 
+}
+
 void
 _wnck_change_state (Screen  *screen,
 		    Window   xwindow,
@@ -897,6 +980,30 @@ _wnck_activate_workspace (Screen *screen,
 	      &xev);
 }
 
+void
+_wnck_toggle_showing_desktop (Screen  *screen,
+                              gboolean show)
+{
+  XEvent xev;
+  
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = DisplayOfScreen (screen);
+  xev.xclient.window = RootWindowOfScreen (screen);
+  xev.xclient.message_type = _wnck_atom_get ("_NET_SHOWING_DESKTOP");
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = show != FALSE;
+  xev.xclient.data.l[1] = 0;
+  xev.xclient.data.l[2] = 0;
+
+  XSendEvent (DisplayOfScreen (screen),
+	      RootWindowOfScreen (screen),
+              False,
+	      SubstructureRedirectMask | SubstructureNotifyMask,
+	      &xev);
+}
+
 Window
 _wnck_get_group_leader (Window xwindow)
 {
@@ -2111,19 +2218,35 @@ _wnck_set_dock_type_hint (Window xwindow)
   _wnck_error_trap_pop ();
 }
 
+/* orientation of pager */
 #define _NET_WM_ORIENTATION_HORZ 0
 #define _NET_WM_ORIENTATION_VERT 1
 
+/* starting corner for counting spaces */
+#define _NET_WM_TOPLEFT     0
+#define _NET_WM_TOPRIGHT    1
+#define _NET_WM_BOTTOMRIGHT 2
+#define _NET_WM_BOTTOMLEFT  3
+
 void
 _wnck_set_desktop_layout (Screen *xscreen,
                           int     rows,
                           int     columns)
 {
-  gulong data[3];
+  gulong data[4];
 
-  data[0] = _NET_WM_ORIENTATION_HORZ;
-  data[1] = rows;
-  data[2] = columns;
+  /* FIXME: hack, hack, hack so as not
+   * to have to add a orientation param
+   * to wnck_screen_try_set_workspace_layout.
+   *
+   * Remove this crack asap.
+   */
+  g_assert ((rows == 0) || (columns == 0));
+
+  data[0] = (columns == 0) ? _NET_WM_ORIENTATION_HORZ : _NET_WM_ORIENTATION_VERT;
+  data[1] = columns;
+  data[2] = rows;
+  data[3] = _NET_WM_TOPLEFT;
   
   _wnck_error_trap_push ();
 
@@ -2131,7 +2254,7 @@ _wnck_set_desktop_layout (Screen *xscreen,
                    RootWindowOfScreen (xscreen),
 		   _wnck_atom_get ("_NET_DESKTOP_LAYOUT"),
 		   XA_CARDINAL, 32, PropModeReplace,
-		   (guchar *)&data, 3);
+		   (guchar *)&data, sizeof (data));
 
   _wnck_error_trap_pop ();
 }
@@ -2346,3 +2469,51 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
 
   return FALSE;
 }
+
+/* stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org> */
+typedef struct
+{
+  char *stock_id;
+  const guint8 *icon_data;
+} StockIcon;
+
+void
+_wnck_stock_icons_init (void)
+{
+  GtkIconFactory *factory;
+  int i;
+  static gboolean done = FALSE;
+
+  StockIcon items[] =
+  {
+    { WNCK_STOCK_DELETE,   stock_delete_data   },
+    { WNCK_STOCK_MINIMIZE, stock_minimize_data },
+    { WNCK_STOCK_MAXIMIZE, stock_maximize_data }
+  };
+
+  if (done)
+    return;
+
+  done = TRUE;
+  
+  factory = gtk_icon_factory_new ();
+  gtk_icon_factory_add_default (factory);
+
+  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
+    {
+      GtkIconSet *icon_set;
+      GdkPixbuf *pixbuf;
+
+      pixbuf = gdk_pixbuf_new_from_inline (-1, items[i].icon_data,
+					   FALSE,
+					   NULL);
+
+      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
+      gtk_icon_set_unref (icon_set);
+		
+      g_object_unref (G_OBJECT (pixbuf));
+    }
+
+  g_object_unref (G_OBJECT (factory));
+}
